@@ -1,4 +1,4 @@
-# $Id: Request.pm 344 2008-12-08 07:23:22Z rcaputo $
+# $Id: Request.pm 365 2009-07-26 23:01:30Z rcaputo $
 
 package POE::Component::Client::HTTP::Request;
 use strict;
@@ -42,14 +42,13 @@ use constant REQ_START_TIME    => 15;
 use constant REQ_FACTORY       => 16;
 use constant REQ_CONN_ID       => 17;
 
-use constant RS_CONNECT        => 0x01;
-use constant RS_SENDING        => 0x02;
-use constant RS_IN_HEAD        => 0x04;
-use constant RS_REDIRECTED     => 0x08;
-use constant RS_IN_CONTENT     => 0x20;
-use constant RS_DONE           => 0x40;
-use constant RS_POSTED         => 0x80;
-
+use constant RS_CONNECT        => 0x01; # establishing a connection
+use constant RS_SENDING        => 0x02; # sending request to server
+use constant RS_IN_HEAD        => 0x04; # waiting for or receiving headers
+use constant RS_REDIRECTED     => 0x08; # request has been redirected
+use constant RS_IN_CONTENT     => 0x20; # waiting for or receiving content
+use constant RS_DONE           => 0x40; # received full content
+use constant RS_POSTED         => 0x80; # we have posted back a response
 
 sub import {
   my ($class) = shift;
@@ -228,7 +227,7 @@ sub add_eof {
       "got " . $self->[REQ_OCTETS_GOT] . " of " .
       $self->[REQ_RESPONSE]->content_length
     );
-    #TODO: shouldn't this be 406 as per RT #20975?
+
     $self->error(
       400,
       "incomplete response b " . $self->[REQ_ID] . ".  Wanted " .
@@ -260,23 +259,29 @@ sub add_content {
   # therein.  If it's done the former, then we're safe.  Otherwise
   # we also need to C<use bytes>.
   # TODO: write test(s) for this.
+
   my $this_chunk_length = length($self->[REQ_BUFFER]);
   $self->[REQ_OCTETS_GOT] += $this_chunk_length;
 
-  my $max = $self->[REQ_FACTORY]->max_response_size;
+  my $max = $self->[REQ_FACTORY]->max_response_size();
 
   DEBUG and warn(
     "REQ: request ", $self->ID,
     " received $self->[REQ_OCTETS_GOT] bytes; maximum is $max"
   );
 
+  # Fail if we've gone over the maximum content size to return.
   if (defined $max and $self->[REQ_OCTETS_GOT] > $max) {
-    # We've gone over the maximum content size to return.  Chop it # back.
-    my $over = $self->[REQ_OCTETS_GOT] - $max;
-    $self->[REQ_OCTETS_GOT] -= $over;
-    substr($self->[REQ_BUFFER], -$over) = "";
-    #TODO: ??
-    #$self->[REQ_STATE] |= RS_DONE;
+    $self->error(
+      406,
+      "Response content is longer than specified MaxSize of $max.  " .
+      "Use range requests to retrieve specific amounts of content."
+    );
+
+    $self->[REQ_STATE] |= RS_DONE;
+    $self->[REQ_STATE] &= ~RS_IN_CONTENT;
+    $self->[REQ_CONNECTION]->close();
+    return 1;
   }
 
   # keep this for the progress callback (it gets cleared in return_response
@@ -301,16 +306,6 @@ sub add_content {
     );
   };
 
-  if (defined $max and $self->[REQ_OCTETS_GOT] >= $max) {
-    DEBUG and warn(
-      "REQ: request ", $self->ID, " has a full response... moving to done."
-    );
-    $self->[REQ_STATE] |= RS_DONE;
-    $self->[REQ_STATE] &= ~RS_IN_CONTENT;
-    $self->[REQ_CONNECTION]->close();
-    return 1;
-  }
-
   if ($self->[REQ_RESPONSE]->content_length) {
 
     # Report back progress
@@ -332,6 +327,7 @@ sub add_content {
       return 1;
     }
   }
+
   return 0;
 }
 
@@ -536,6 +532,7 @@ sub error {
     . "<BODY>$nl"
     . "<H1>Error: $http_msg</H1>$nl"
     . "$message$nl"
+    . "<small>This is a client error, not a server error.</small>$nl"
     . "</BODY>$nl"
     . "</HTML>$nl"
   );
