@@ -1,4 +1,4 @@
-# $Id: HTTPHead.pm 285 2006-10-14 06:04:39Z rcaputo $
+# $Id: HTTPHead.pm 365 2009-07-26 23:01:30Z rcaputo $
 
 package POE::Filter::HTTPHead_Line;
 use warnings;
@@ -55,8 +55,9 @@ sub get_one {
       }
       else {
         # assume HTTP/0.9
-        my $resp = HTTP::Response->new ('200', 'OK', 
-                    ['Content-Type' => 'text/html'], $line );
+        my $resp = HTTP::Response->new (
+          '200', 'OK', ['Content-Type' => 'text/html'], $line
+        );
         $resp->protocol('HTTP/0.9');
         return [ $resp ];
       }
@@ -87,7 +88,7 @@ sub get_one {
         $
         }x
       ) {
-        $self->[WORK_RESPONSE]->header($1, $2)
+        $self->[WORK_RESPONSE]->push_header($1, $2)
       }
     }
   }
